package pack
{
	import flash.display.*;
	import flash.events.*;
	import flash.utils.*;
	
	import mx.core.UIComponent;
	
	public class Arkanoid extends UIComponent
	{
		// Definir los parámetros del juego
		var NRO_BLOQUES_X_FILA:int = 6;
		var NRO_BLOQUES_X_COLUMNA:int = 12;
		
		var RADIO_PELOTA:int = 4;
		var VELOCIDAD_INICIAL_PELOTA = 6;
		
		var BLOCK_FIELD_Y = 50;
		var BLOCK_GUTTER_X = 2;
		var BLOCK_GUTTER_Y = 2;
		
		var PADDLE_HEIGHT:int = 15;
		var PADDLE_WIDTH:int = 50;
		var PADDLE_DIST_FROM_BOTTOM:int = 25;
		
		// Sprites
		private var bloques:Array = new Array(0);
		private var pelota:Sprite = new Sprite();
		private var paddle:Sprite = new Sprite();
		
		// Variables del estado del juego      
		private var pelota_dx:Number;
		private var pelota_dy:Number;
		private var pelota_speed:Number;
		private var vidas:int = 3;
		
		// Insertar gráficos en el archivo .swf (solo funciona en Flex)
		[Embed(source="graficos/gold.jpg")]
		private var GoldBlock:Class;
		[Embed(source="graficos/rojo.jpg")]
		private var RedBlock:Class;
		[Embed(source="graficos/blue.jpg")]
		private var BlueBlock:Class;
		
		// Se guardan las imágenes en el array
		private var todosLosBloques:Array = new Array(GoldBlock, RedBlock, BlueBlock);
		
		public function init():void
		{
			
			// pelota
			pelota.graphics.beginFill(0x2e0fe3);
			pelota.graphics.drawCircle(0, 0, RADIO_PELOTA);
			pelota.graphics.endFill();
			addChild(pelota);
			resetpelota(); // se ubica la pelota en la posición inicial
			
			// paddle
			paddle.graphics.beginFill(0x000000);
			paddle.graphics.drawRect(-PADDLE_WIDTH / 2, -PADDLE_HEIGHT / 2, PADDLE_WIDTH, PADDLE_HEIGHT);
			paddle.graphics.endFill();
			paddle.y = height - PADDLE_DIST_FROM_BOTTOM;
			addChild(paddle);
			
			// 
			var reference_block:DisplayObject = new GoldBlock();
			
			var block_field_width:int = NRO_BLOQUES_X_COLUMNA * reference_block.width + (NRO_BLOQUES_X_COLUMNA - 1) * BLOCK_GUTTER_X;
			var block_field_x:int = (width - block_field_width) / 2;
			
			var current_block_index:int = 0;
			
			// Ubicando los bloques
			for (var x:int = 0; x < NRO_BLOQUES_X_COLUMNA; x++)
			{
				for (var y:int = 0; y < NRO_BLOQUES_X_FILA; y++)
				{
					// Se crea un bloque
					var current_image:Class = todosLosBloques[current_block_index];
					var sprite:DisplayObject = new current_image();
					
					sprite.x = block_field_x + x * (reference_block.width + BLOCK_GUTTER_X);
					sprite.y = BLOCK_FIELD_Y + y * (reference_block.height + BLOCK_GUTTER_Y);
					
					addChild(sprite);
					
					bloques.push(sprite);
					
					current_block_index++;
					if (current_block_index >= todosLosBloques.length)
						current_block_index = 0;
				}
			}
			
			// Se crea el timer
			var ticker = new Timer(10);
			ticker.addEventListener(TimerEvent.TIMER, onTick);
			ticker.start();
		}
		
		public function onTick(evt:TimerEvent):void
		{
			// Se ubica el paddle
			paddle.x = mouseX;
			
			// Colisión paddle y pelota
			if (pelota.hitTestObject(paddle))
			{
				// Adjust X momentum of pelota based on where on the paddle we hit
				var new_pelota_dx:Number = (pelota.x - paddle.x) / (PADDLE_WIDTH / 2.0) * VELOCIDAD_INICIAL_PELOTA;
				
				// Blend this new momentum with the old
				pelota_dx = (pelota_dx + new_pelota_dx) / 2;
				pelota_dy = -pelota_dy;
			}
			
			// Se chequean colisiones con cada uno de los bloques
			for (var i:int = 0; i < bloques.length; i++)
			{
				var block:DisplayObject = bloques[i];
				if (block == null)
					continue;
				
				// Check collision
				if (pelota.hitTestObject(block))
				{
					// Si colisiona se borra el bloque
					removeChild(block);
					bloques[i] = null;
					
					var pelota_distance_norm_x:int = (pelota.x - block.x) / block.width;
					var pelota_distance_norm_y:int = (pelota.y - block.y) / block.height;
					
					if (pelota_distance_norm_x * pelota_dx > 0)
						pelota_dy = -pelota_dy;
					else if (pelota_distance_norm_y * pelota_dy > 0)
						pelota_dx = -pelota_dx;
					
					else if (pelota_distance_norm_x > pelota_distance_norm_y)
						pelota_dx = -pelota_dx;
					else
						pelota_dy = -pelota_dy;
					
					// Se aumetna la velocidad de la pelota 2.5% 
					pelota_dx *= 1.025;
					pelota_dy *= 1.025;
				}
			}
			
			// Colisiones de los bordes
			if (pelota.x - RADIO_PELOTA < 0)
				pelota_dx = Math.abs(pelota_dx);
			if (pelota.x + RADIO_PELOTA > width)
				pelota_dx = -Math.abs(pelota_dx);
			if (pelota.y - RADIO_PELOTA < 0)
				pelota_dy = Math.abs(pelota_dy);
			
			if (pelota.y - RADIO_PELOTA > height)
			{
				if (vidas > 0)
					vidas -= 1;
				resetpelota();
				
			}
			
			// Actualizar pelota
			pelota.x += pelota_dx;
			pelota.y += pelota_dy;
		
		}
		
		public function resetpelota():void
		{
			pelota.x = .25 * width;
			pelota.y = .5 * height;
			pelota_dx = VELOCIDAD_INICIAL_PELOTA * .70;
			pelota_dy = VELOCIDAD_INICIAL_PELOTA * .70;
		}
	}
}
//Fuente http://sourcecookbook.com/es/recipes/1/arkanoid-super-sencillo-en-as-3-0-compilable-con-mxmlc-compilador-flex